/* ------------------------------------------------------------------- 
* Header files 
* ------------------------------------------------------------------- */ 
#include "device.h"

/* ------------------------------------------------------------------- 
* Global Functions 
* ------------------------------------------------------------------- */ 
void ConfigureClocks(void);
void CacheClear(void);
void CacheBypass(void);
void CacheEnablePrefetch(void);
void Set_MemMap_XIP(void);
void spi_send_bytes(uint8_t * pData, uint32_t nNumBytes);
void spi_exit_xip(void);

/* ------------------------------------------------------------------- 

* ------------------------------------------------------------------- */ 
/*
    This function configures the following :

      CGU - FUll ON and CCLK/SCLK = 240 MHz / 96 MHz or 225 MHz / 100 MHz or 200 MHz / 100 MHz or 240 MHz / 80 MHz
      SPI - After boot, most settings can be reused
            Re-configure only Baud Rate 
*/
void ConfigureClocks()
{
        #ifdef CCLK240_SCLK96  
        /*Set CGU for CCLK/SCLK = 240 MHz / 96 MHz */
	Init_PLL(32, 1, 2, 5);
        #endif    
        #ifdef CCLK225_SCLK100   
        /*Set CGU for CCLK/SCLK = 225 MHz / 100 MHz */
	Init_PLL(30, 0, 4, 9);
        #endif   
        #ifdef CCLK200_SCLK100   
        /*Set CGU for CCLK/SCLK = 200 MHz / 100 MHz */
	Init_PLL(20, 0, 3, 6);
        #endif 
        #ifdef CCLK240_SCLK80   
        /*Set CGU for CCLK/SCLK = 240 MHz / 80 MHz */
	Init_PLL(32, 1, 2, 6);
        #endif
              
        *pREG_SPI2_CLK = 0;
        
         __DSB();
}

/*
    This function sets up bypassing of cache for further access  
*/
void ByPassCache()
{
    *pREG_M4P0_CACHE_CFG |= 0x3;
    __DSB();
}

/*
    This function enables prefetch in the cache  
*/
void CacheEnablePrefetch()
{
    *pREG_M4P0_CACHE_CFG |= ((7 << BITP_M4P_CACHE_CFG_PF ) & BITM_M4P_CACHE_CFG_PF );
}

/*      
   This function clears the cache memory completely, 
   so that all further accesses are cache Miss
*/
void CacheClear()
{
    //Set CLEAR bit
    *pREG_M4P0_CACHE_CFG |= BITM_M4P_CACHE_CFG_CLEAR;
    __DSB();
      
    //wait for CLEAR bit to be self-cleared
    while(*pREG_M4P0_CACHE_CFG & BITM_M4P_CACHE_CFG_CLEAR)__NOP();      
    __DSB();    
    
}

void Set_MemMap_XIP()
{
    
  int temp=0;
  unsigned int* pSPI_MEM = (unsigned int*) 0x18004000;
  
  *pREG_SPI2_CTL = 0;                           /* Disable the SPI */     
  *pREG_SPI2_CTL = 0x3;                         /* Putting the SPI into single bit mode and enabling SPI again*/

  spi_exit_xip();                           /* Remove fron XIP mode */
  
 /* Disable SPI and make sure we are out of Memory-mapped mode */
  *pREG_SPI2_CTL = 0;  __DSB();
  *pREG_SPI2_MMRDH = 0;  __DSB();

/* Program baud rate */
   *pREG_SPI2_CLK = 0x00;
        
/* Program delay register with lead and lag times */
   *pREG_SPI2_DLY=(((3 << BITP_SPI_DLY_STOP ) & BITM_SPI_DLY_STOP ) |
                    ((1 << BITP_SPI_DLY_LEADX) & BITM_SPI_DLY_LEADX) |
                    ((1 << BITP_SPI_DLY_LAGX)  & BITM_SPI_DLY_LAGX ) );
	 
/* Program SPI control register */
   *pREG_SPI2_CTL = (ENUM_SPI_CTL_MM_EN | ENUM_SPI_CTL_MASTER | 
		     ENUM_SPI_CTL_SIZE32 | ENUM_SPI_CTL_HW_SSEL | 
		     ENUM_SPI_CTL_ASSRT_SSEL | ENUM_SPI_CTL_MSB_FIRST | 
		     ENUM_SPI_CTL_FAST_EN | ENUM_SPI_CTL_MIO_QUAD |
		     ((0 << BITP_SPI_CTL_CPHA) & BITM_SPI_CTL_CPHA)|
		     ((0 << BITP_SPI_CTL_CPOL) & BITM_SPI_CTL_CPOL));

/* Program transmit control register */
   *pREG_SPI2_TXCTL = ( BITM_SPI_TXCTL_TTI | BITM_SPI_TXCTL_TEN );  
/* Program receive control register */
   *pREG_SPI2_RXCTL = ( BITM_SPI_RXCTL_REN );

/* Program Memory Map Read Header register */
   *pREG_SPI2_MMRDH = (((0xEB << BITP_SPI_MMRDH_OPCODE ) & BITM_SPI_MMRDH_OPCODE)|
		      ((3 << BITP_SPI_MMRDH_ADRSIZE) & BITM_SPI_MMRDH_ADRSIZE)|
		      ((1 << BITP_SPI_MMRDH_ADRPINS) & BITM_SPI_MMRDH_ADRPINS)|
		      ((5 << BITP_SPI_MMRDH_DMYSIZE) & BITM_SPI_MMRDH_DMYSIZE)|
		      ((0xFF << BITP_SPI_MMRDH_MODE) & BITM_SPI_MMRDH_MODE)|
		      ((1 << BITP_SPI_MMRDH_TRIDMY) & BITM_SPI_MMRDH_TRIDMY)|
		      ((1 << BITP_SPI_MMRDH_MERGE) & BITM_SPI_MMRDH_MERGE)|
		      ((0 << BITP_SPI_MMRDH_CMDSKIP) & BITM_SPI_MMRDH_CMDSKIP)|	 
		      ((0 << BITP_SPI_MMRDH_CMDPINS) & BITM_SPI_MMRDH_CMDPINS));
	
/* Program Memory-mapped Top register */
   *pREG_SPI2_MMTOP = 0x18000000 + (0x10000 * 32); 
   
/* Program SPI with slave select */
   *pREG_SPI2_SLVSEL = ENUM_SPI_SLVSEL_SSEL1_HI | ENUM_SPI_SLVSEL_SSEL1_EN;   
   __DSB();
 
/* Now enable SPI */
   *pREG_SPI2_CTL |= BITM_SPI_CTL_EN;
   __DSB();  
  
  /* Setting XIP mode bits */
  
  *pREG_SPI2_MMRDH = (*pREG_SPI2_MMRDH & (~BITM_SPI_MMRDH_MODE)) | ((0x20 << BITP_SPI_MMRDH_MODE) & BITM_SPI_MMRDH_MODE);
  
  temp = *pSPI_MEM;                             /* Dummy read through pointer */
   
  *pREG_SPI2_MMRDH |= BITM_SPI_MMRDH_CMDSKIP;
    
}

void spi_exit_xip()
{
  /* send Continuous Read Mode Reset (FFh or FFFFh) */
  uint32_t nSlvSelHigh = ((BITM_SPI_SLVSEL_SSE1 << 0) | (0xFF00));
  uint32_t nSlvSelLow = ((BITM_SPI_SLVSEL_SSE1 << 0) | (0xFF00 & ~(BITM_SPI_SLVSEL_SSEL1 << 0)));
  uint8_t spiData[1];

  spiData[0] = 0xFF;

  *pREG_SPI2_SLVSEL = nSlvSelLow; /* set slave select low */
  *pREG_SPI2_SLVSEL = nSlvSelLow; /* set slave select low */
  spi_send_bytes(&spiData[0], 1);

  *pREG_SPI2_SLVSEL = nSlvSelHigh; /* set slave select high */
  *pREG_SPI2_SLVSEL = nSlvSelHigh; /* set slave select high */
}

void spi_send_bytes(uint8_t * pData, uint32_t nNumBytes)
{
  uint32_t i;

  if((nNumBytes != 0))
  {
    uint32_t txctl = *pREG_SPI2_TXCTL;
    *pREG_SPI2_RXCTL = 0;

    *pREG_SPI2_TWC = nNumBytes;
    *pREG_SPI2_TXCTL = BITM_SPI_TXCTL_TEN | BITM_SPI_TXCTL_TTI | BITM_SPI_TXCTL_TWCEN;

    for(i=0; i<nNumBytes; i++)
    {
      *pREG_SPI2_TFIFO = (pData[i]);
      while((*pREG_SPI2_STAT & BITM_SPI_STAT_TFS) == ENUM_SPI_STAT_TFIFO_FULL);  /* Wait until there is space in the FIFO */
    }

    while((*pREG_SPI2_STAT & BITM_SPI_STAT_TF) == 0);  /* Wait until finished */
    *pREG_SPI2_STAT = BITM_SPI_STAT_TF;
    *pREG_SPI2_TXCTL = txctl & (~BITM_SPI_TXCTL_TEN);
  }
}

