/* ------------------------------------------------------------------- 
* Header files 
* ------------------------------------------------------------------- */ 
#include "device.h"
#include "Memory_placement.h"

/* ------------------------------------------------------------------- 
* MACROs
* ------------------------------------------------------------------- */ 
#if 1// switch between 408 & 403 EZ-KITs
// LED2 macros for 408 board (PortE, bit6)
#define LED2_PORT    pADI_PORTE
#define LED2_PIN     0x40
#else
// LED2 macros for 403 board (PortB, bit4)
#define LED2_PORT    pADI_PORTB
#define LED2_PIN     0x10
#endif

#define LED2_ON      LED2_PORT->DATA_SET
#define LED2_OFF     LED2_PORT->DATA_CLR


#define REG_DWT0_CYCCNT                 0xE0001004                                               /*  DWT0 DWT Current PC Sampler Cycle Count Register */
#define pREG_DWT0_CYCCNT                ((volatile  uint32_t *) REG_DWT0_CYCCNT)                 /*  DWT Current PC Sampler Cycle Count Register */
#define	START_COUNT()			tick_before = *pREG_DWT0_CYCCNT;
#define STOP_COUNT()			tick_after = *pREG_DWT0_CYCCNT;

/* ------------------------------------------------------------------- 
* Global Functions 
* ------------------------------------------------------------------- */ 

void LED_Blink(int BlinkCnt);
void CallSomeFlashCode(int BlinkCnt);

/* ---------------------------------------------------------------------- 
* Flash based LED blink code 
* ------------------------------------------------------------------- */ 
int main()
{
    /* system initialization with CMSIS API*/
    SystemInit(); 
    
    #ifdef PLACE_ASYNCSRAM
    DoAsyncInit();
    #endif
    #ifdef PLACE_MEMYCODE
    DoMEMYInit();
    #endif   
     
    /* configures CGU */
    #ifdef SET_CLOCKS
    ConfigureClocks();
    #endif
    
    /* Configures the flash in XIP mode */
    #ifdef PLACE_FLASH 
      #ifdef XIP_ENABLE
        Set_MemMap_XIP();
      #endif
    #endif
    
    /* Configure cache with or without prefetch enabled */
    #ifndef BYPASS_CACHE
      #ifdef PREFETCH_ENABLE
        CacheEnablePrefetch();
      #endif
    #endif
    
    #ifdef BYPASS_CACHE
    CacheBypass();
    #endif          
    
    #ifdef RUN_PERF_ANALYSIS
    CallPerfAnalysis();
    #else
    /*call the blink function inside flash*/
    LED2_PORT->DIR_SET = LED2_PIN;
    CallSomeFlashCode(300000);
    #endif
      
    return 0;
}

#ifdef PLACE_MEMYCODE
  #pragma location = ".memy_code"
#endif
#ifdef PLACE_ASYNCSRAM
  #pragma location = ".async_code"
#endif
#ifdef PLACE_FLASH
  #pragma location = ".flash_code"
#endif
void LED_Blink( int BlinkCnt )
{           
         int Cnt;

         while(1)
     {           
    
            LED2_ON = LED2_PIN;
            for(Cnt=0;Cnt<BlinkCnt/2;Cnt++);
                      
            LED2_OFF = LED2_PIN;
            for(Cnt=0;Cnt<BlinkCnt/2;Cnt++);
            
      }

}

/*
    This is the function that is placed inside Internal SPI Flash
    defining MACRO PLACE_FLASH places this on to flash memory
*/
#ifdef PLACE_FLASH
  #pragma location = ".flash_code"
#endif
#ifdef PLACE_ASYNCSRAM
  #pragma location = ".async_code"
#endif
#ifdef PLACE_MEMYCODE
  #pragma location = ".memy_code"
#endif
void CallSomeFlashCode(int BlinkCnt)
{
  BlinkCnt *= 5;
  LED_Blink(BlinkCnt);
   
}


